{-----------------------------------------------------------------------------
 GLPlanet!
 Author: Alexandre Hirzel
 Modified By:  Aaron Hochwimmer
 04/07/2003: Alexandre Hirzel: Added time-dependent sun position
             Time zone function provided by Tobias Peirick
             DateSeparator bug fixed (thanks to Philipp)
 
 Purpose: Shows GLScene users and developers around the world!
 Eric Grange (GLScene maintainer) is a 'type 0' so is displayed in a
 different colour!

 Format of location file entry (where date is in dd/mm/yyyy)
 longitude latitude type date
-----------------------------------------------------------------------------}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GLScene, GLObjects, GLMisc, GLWin32Viewer, GLCadencer, GLTexture,
  jpeg, Geometry, GLSkydome, Menus, ComCtrls, ToolWin, ExtCtrls;

type
  TMarkerPosition = class(TObject)
  private
    fLatitude : single;
    fLongitude : single;
    fmembertype : byte; // 0..255
    fDateAdded : TDate;
  public
    function GetCartesian(dRadius:single):TVector;
    property DateAdded : TDate read fDateAdded write fDateAdded;
    property membertype :byte read fmembertype write fmembertype;
    property Latitude : single read fLatitude write fLatitude;
    property Longitude : single read fLongitude write fLongitude;
  end;

  TForm1 = class(TForm)
    GLSceneViewer1: TGLSceneViewer;
    GLScene1: TGLScene;
    sphEarth: TGLSphere;
    GLCamera1: TGLCamera;
    lsSun: TGLLightSource;
    GLCadencer1: TGLCadencer;
    GLMaterialLibrary1: TGLMaterialLibrary;
    ptsLocations: TGLPoints;
    sdSky: TGLSkyDome;
    MainMenu1: TMainMenu;
    miFile: TMenuItem;
    miExit: TMenuItem;
    miSpinThePlanet: TMenuItem;
    N1: TMenuItem;
    miMarkers: TMenuItem;
    miRound: TMenuItem;
    miSmooth: TMenuItem;
    miSmoothAdditive: TMenuItem;
    miSquare: TMenuItem;
    ToolBar1: TToolBar;
    DateTimePicker1: TDateTimePicker;
    timUpdateSun: TTimer;
    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure miSpinThePlanetClick(Sender: TObject);
    procedure GLSceneViewer1MouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewer1MouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure GLSceneViewer1MouseEnter(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure miRoundClick(Sender: TObject);
    procedure miSmoothClick(Sender: TObject);
    procedure miSmoothAdditiveClick(Sender: TObject);
    procedure miSquareClick(Sender: TObject);
    procedure DateTimePicker1Change(Sender: TObject);
    procedure timUpdateSunTimer(Sender: TObject);
  private
    mdx, mdy : Integer;
    markers : TStringList;
    procedure ClearLocations;
    procedure DrawPoints;
    procedure SunPosition(const GMTTime:TTime);
    function GMTNow:tTime;
    { Private declarations }
  public
  
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// ----- TMarkerPosition.GetCartesian ------------------------------------------
{Convert from Longitude-Latitude system to cartesian coordinates.
 Correct for offset.
 Longitude are positive eastward and latitude northward}
function TMarkerPosition.GetCartesian(dRadius:single):TVector;

var
  ca,sa,co,so	:single;

begin
  SinCos(DegToRad(-Longitude-90),so,co); // 90 offset
  SinCos(DegToRad(Latitude),sa,ca);

  Result[0]:=dRadius*co*ca;
  Result[1]:=dRadius*sa;
  Result[2]:=dRadius*so*ca;
end;
// ----- TForm1.GLCadencer1Progress --------------------------------------------
procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime,
  newTime: Double);
begin
  if miSpinThePlanet.Checked then sphEarth.TurnAngle:=15*newTime;
end;
// ----- TForm1.FormCreate -----------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
var
  Lat,Lon	:single;
  F			:TextFile;
  mp 		: TMarkerPosition;
  iIndex 	: integer;
  iType 		: byte;
  sDate,sDateFormat : string;
  cDateSeparator: char;
begin
  markers := TStringList.CReate;

  {Earth image from http://www.sulaco.co.za/}
  GLMaterialLibrary1.AddTextureMaterial('Earth','Earth.jpg');
  sphEarth.Material.MaterialLibrary:=GLMaterialLibrary1;
  sphEarth.Material.LibMaterialName:='Earth';

  sdSky.Stars.AddRandomStars(1000,clWhite);

{Load coordinate list}

  sDateFormat := ShortDateFormat; // save it
  cDateSeparator := DateSeparator;
  DateSeparator := '/'; // that from the locations.txt

  ShortDateFormat := 'dd/mm/yyyy';
  AssignFile(F,'Locations.txt');
  Reset(F);
  try
    ClearLocations;
    iIndex := 0;
    while not eof(F) do begin
      Readln(F,Lon,Lat,iType,sDate);
      mp := Tmarkerposition.Create;
      markers.AddObject(IntToStr(iIndex),mp);
      with TMarkerPosition(markers.Objects[iIndex]) do
      begin
        Latitude := Lat;
        Longitude := Lon;
        MemberType := iType;
        DateAdded := StrToDate(Trim(sDate));
      end;
      Inc(iIndex);
    end;//while
  finally
    DateSeparator := cDateSeparator;
    CloseFile(F);
  end;//finally

  ShortDateFormat := sDateFormat;
  DrawPoints;
  DateTimePicker1.DateTime := Now;

  timUpdateSunTimer(Sender); // Initialize sun position
end;
// ----- TForm1.FormDestroy ----------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ClearLocations;
  markers.Free;
end;
// ----- Tform1.ClearLocations -------------------------------------------------
procedure Tform1.ClearLocations;

begin
  while markers.Count > 0 do
  begin
    TMarkerPosition(markers.Objects[0]).Free;
    markers.Delete(0);
  end;
  markers.Clear;
end;
// ----- Tform1.DrawPoints -----------------------------------------------------
procedure Tform1.DrawPoints;

var
  i:integer;

begin
  ptsLocations.Positions.Clear;
  ptsLocations.Colors.Clear;

  for i := 0 to markers.Count-1 do
  begin
// only add dates prior to display date
    if (TMarkerPosition(markers.Objects[i]).DateAdded < DateTimePicker1.DateTime) then
    begin
      case TMarkerPosition(markers.Objects[i]).membertype of
        0: ptsLocations.Colors.Add(clryellow); // Eric has his own type!
        1: ptsLocations.Colors.Add(clrRed); // general
      end;
      ptsLocations.Positions.Add(TMarkerPosition(markers.Objects[i]).
        GetCartesian(sphEarth.Radius));
    end;
  end;
  ptsLocations.StructureChanged;
end;
// ----- TForm1.miExitClick ----------------------------------------------------
procedure TForm1.miExitClick(Sender: TObject);
begin
  Close;
end;
// ----- TForm1.miSpinThePlanetClick -------------------------------------------
procedure TForm1.miSpinThePlanetClick(Sender: TObject);
begin
  GLCadencer1.Enabled := miSpinThePlanet.Checked; // on autocheck
end;
// ----- TForm1.GLSceneViewer1MouseDown ----------------------------------------
procedure TForm1.GLSceneViewer1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	// store mouse coordinates when a button went down
	mdx:=x; mdy:=y;
end;
// ----- TForm1.GLSceneViewer1MouseMove ----------------------------------------
procedure TForm1.GLSceneViewer1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
	dx, dy : Integer;
	v : TVector;
begin
	// calculate delta since last move or last mousedown
	dx:=mdx-x; dy:=mdy-y;
	mdx:=x; mdy:=y;
	if ssLeft in Shift then
    GLCamera1.MoveAroundTarget(dy, dx)
end;
// ----- TForm1.GLSceneViewer1MouseEnter ---------------------------------------
procedure TForm1.GLSceneViewer1MouseEnter(Sender: TObject);
begin
  GLSceneViewer1.Focused;  
end;
// ----- TForm1.FormMouseWheel -------------------------------------------------
procedure TForm1.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
	// Note that 1 wheel-step induces a WheelDelta of 120,
	// this code adjusts the distance to target with a 10% per wheel-step ratio
	GLCamera1.AdjustDistanceToTarget(Power(1.1, WheelDelta/120));
end;

procedure TForm1.miRoundClick(Sender: TObject);
begin
  miRound.Checked := true;
  miSmooth.Checked := not miRound.CHecked;
  miSmoothAdditive.Checked := not miRound.Checked;
  miSquare.Checked := not miRound.Checked;
  ptsLocations.Style := psRound;
end;
// ----- TForm1.miSmoothClick --------------------------------------------------
procedure TForm1.miSmoothClick(Sender: TObject);
begin
  miSmooth.Checked := true;
  miRound.Checked := not miSmooth.CHecked;
  miSmoothAdditive.Checked := not miSmooth.Checked;
  miSquare.Checked := not miSmooth.Checked;
  ptsLocations.Style := psSmooth;
end;
// ----- TForm1.miSmoothAdditiveClick ------------------------------------------
procedure TForm1.miSmoothAdditiveClick(Sender: TObject);
begin
  miSmoothAdditive.Checked := true;
  miRound.Checked := not miSmoothAdditive.Checked;
  miSmooth.Checked := not miSmoothAdditive.Checked;
  miSquare.Checked := not miSmoothAdditive.Checked;
  ptsLocations.Style := psSmoothAdditive;
end;
// ----- TForm1.miSquareClick --------------------------------------------------
procedure TForm1.miSquareClick(Sender: TObject);
begin
  miSquare.Checked := true;
  miRound.Checked := not miSquare.CHecked;
  miSmoothAdditive.Checked := not miSquare.Checked;
  miSmooth.Checked := not miSquare.Checked;
  ptsLocations.Style := psSquare;
end;
// ----- TForm1.DateTimePicker1Change ------------------------------------------
procedure TForm1.DateTimePicker1Change(Sender: TObject);
begin
  DrawPoints; // redraw points based on new date constraint
end;

procedure TForm1.SunPosition(const GMTTime: TTime);
var
  Angle	:single;
begin
  Angle:=Frac(GMTTime)*2*pi+pi/2;
  lsSun.SpotDirection.SetVector(Cos(Angle),0,Sin(Angle));
end;

procedure TForm1.timUpdateSunTimer(Sender: TObject);
begin
  SunPosition(GMTNow);
end;

function TForm1.GMTNow: tTime;
{ Function provided by Tobias Peirick }
var
  systime : TSystemTime;
  tz_info : TTimeZoneInformation;
begin
  GetTimeZoneInformation(tz_info);
  Result := Now + tz_info.Bias / 60 /24;
end;

end.

